/*	pam.c	19-June-86
	copyright (c) 1987 William R. Pearson
	read in the alphabet and pam matrix data
	designed for universal matcher

	This version reads BLAST format (square) PAM files
*/

/* $Name: fa33t07_b_d0 $ - $Id: apam.c,v 1.5.4.1 2000/10/13 21:58:23 wrp Exp $ */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "defs.h"
#include "param.h"

#define XTERNAL
#include "uascii.h"
#include "upam.h"
#undef XTERNAL

initpam (mfname, ppst)
char   *mfname;
struct pstruct *ppst;
{
   char    line[512], *lp;
   int     i, iaa, ipam;
   int *hsq, nsq;
   char *sq;
   int ess_tmp;
   FILE   *fmat;

   if ((fmat = fopen (mfname, "r")) == NULL)
   {
      printf (" cannot open scoring matrix file %s\n", mfname);
      return 0;
   }

   hsq = ppst->hsq;
   nsq = ppst->nsq;
   sq = ppst->sq;

   while (fgets (line, sizeof(line), fmat) != NULL && line[0]=='#');

   ess_tmp = sascii[','];

/*	clear out sascii	*/
   for (i = 0; i <= AAMASK; i++) sascii[i] = NA;

/*	set end of line stop	*/
   sascii[0] = sascii['\r'] = sascii['\n'] = EL;

/*	set end of sequence stop */
   sascii['*'] = ES;
   sascii[','] = ess_tmp;

/* read the alphabet */
   sq[0] = '\0';
   for (i = 0, nsq = 1; line[i]; i++) {
     if (line[i] == '*') continue;
     if (line[i] > ' ') sq[nsq++] = toupper (line[i]);
   }
   sq[nsq]='\0';
   nsq--;
   ppst->nsq = nsq;
   fprintf(stderr,"sq[%d]: %s\n",nsq,sq+1);

/* initialize sascii */
   for (iaa = 1; iaa <= nsq; iaa++) {
     sascii[sq[iaa]] = iaa;
     if (sascii[aa[iaa]] < NA && sq[iaa] >= 'A' && sq[iaa] <= 'Z')
       sascii[aa[iaa] - 'A' + 'a'] = sascii[aa[iaa]];
   }
   if (ppst->dnaseq==1) sascii['U'] = sascii['u'] = sascii['T'];

/* read in hnt values */
   hsq[0] = 0;
   for (iaa = 1; iaa <= nsq; iaa++) {
     hsq[iaa]=iaa;
   }
   if (ppst->dnaseq==1) {
     hsq[sascii['R']]=hsq[sascii['M']]=hsq[sascii['W']]=hsq[sascii['A']];
     hsq[sascii['D']]=hsq[sascii['H']]=hsq[sascii['V']]=hsq[sascii['A']];
     hsq[sascii['N']]=hsq[sascii['X']]=hsq[sascii['A']];
     hsq[sascii['Y']]=hsq[sascii['S']]=hsq[sascii['B']]=hsq[sascii['C']];
     hsq[sascii['K']]=hsq[sascii['G']];
   }
   else {
     hsq[sascii['B']] = hsq[sascii['N']];
     hsq[sascii['Z']] = hsq[sascii['E']];
     hsq[sascii['X']] = hsq[sascii['A']];
   }

   for (iaa = 1, ipam = 0; iaa <= nsq; iaa++) {
     if (fgets(line,sizeof(line),fmat)==NULL) {
       fprintf (stderr," error reading pam line: %s\n",line);
       exit (1);
     }
     /*     fprintf(stderr,"%d/%d %s",iaa,nsq,line); */
     strtok(line," \t\n");
     for (i = 0; i < iaa; i++,ipam++) {
       lp=strtok(NULL," \t\n");
       pam[ipam]=atoi(lp);
     }
   }

   /*
   for (iaa = 1, ipam = 0; iaa <= nsq; iaa++) {
     fprintf(stderr,"%c",sq[iaa]);
     for (i=0; i< iaa; i++) 
       fprintf(stderr," %2d",pam[ipam++]);
     fprintf(stderr,"\n");
   }
   */

   ppst->nsq = nsq;
   strncpy (ppst->pamfile, mfname, 40);
   fclose (fmat);
   return 1;
}

/* make a DNA scoring from +match/-mismatch values */

void mk_n_pam(int *arr,int siz, int mat, int mis)
{
  int i, j, k;
  /* current default match/mismatch values */
  int max_mat = +5;
  int min_mis = -4;
  float f_val, f_scale;
  
  f_scale = (float)(mat - mis)/(float)(max_mat - min_mis);
  
  k = 0;
  for (i = 0; i<nnt-1; i++)
    for (j = 0; j <= i; j++ ) {
      if (arr[k] == max_mat) arr[k] = mat;
      else if (arr[k] == min_mis) arr[k] = mis;
      else if (arr[k] != -1) { 
	f_val = (arr[k] - min_mis)*f_scale + 0.5;
	arr[k] = f_val + mis;
      }
      k++;
    }
}

void
init_ascii(char *sq, int nsq, int *sascii, int dnaseq) {

  int i, iaa, ess_tmp, have_lc;

  ess_tmp = sascii[','];

/*	clear out sascii	*/
  for (i = 0; i <= AAMASK; i++) sascii[i] = NA;

/*	set end of line stop	*/
  sascii[0] = sascii['\r'] = sascii['\n'] = EL;

/*	set end of sequence stop */
  sascii['*'] = ES;
  sascii[','] = ess_tmp;

/* initialize sascii from sq[], checking for lower-case letters */
  have_lc = 0;
  for (iaa = 1; iaa <= nsq; iaa++) {
     sascii[sq[iaa]] = iaa;
     if (sq[iaa] >= 'a' && sq[iaa] <= 'z') have_lc = 1;
  }

  if (dnaseq==1) sascii['U'] = sascii['T'];

  /* no lower case letters in alphabet, map lower case to upper */
  if (have_lc != 1) { 
    for (iaa = 1; iaa <= nsq; iaa++) {
      if (sq[iaa] >= 'A' && sq[iaa] <= 'Z') sascii[sq[iaa]-'A'+'a'] = iaa;
    }
    if (dnaseq==1) sascii['u'] = sascii['T'];
  }
}
